package cn.com.qmth.examcloud.starters.face.verify.service.baidu;

import cn.com.qmth.examcloud.starters.face.verify.common.HttpClientBuilder;
import cn.com.qmth.examcloud.starters.face.verify.common.JsonHelper;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

public class BaiduApiHelper {

    private final static Logger log = LoggerFactory.getLogger(BaiduApiHelper.class);

    public static BaiduSession getAccessToken(String apiKey, String apiSecret, String apiUrl) {
        if (StringUtils.isBlank(apiKey)) {
            throw new IllegalArgumentException("Baidu apiKey must be not empty.");
        }

        if (StringUtils.isBlank(apiSecret)) {
            throw new IllegalArgumentException("Baidu apiSecret must be not empty.");
        }

        if (StringUtils.isBlank(apiUrl)) {
            throw new IllegalArgumentException("Baidu apiUrl must be not empty.");
        }

        String requestUrl = String.format("%s?grant_type=client_credentials&client_id=%s&client_secret=%s", apiUrl, apiKey, apiSecret);
        Request.Builder request = new Request.Builder().url(requestUrl).get();

        BaiduSession result = null;
        try (Response response = HttpClientBuilder.getClient().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();

            if (response.isSuccessful()) {
                result = new JsonHelper().parseJson(bodyStr, BaiduSession.class);
            } else {
                log.warn("[baidu] url:{} response:{} body:{}", requestUrl, response.code(), bodyStr);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        if (result == null) {
            throw new RuntimeException("获取 Baidu AccessToken 失败！");
        }

        return result;
    }

    public static BaiduResponse faceDetect(String accessToken, String apiUrl, String imageUrl) {
        if (StringUtils.isBlank(apiUrl)) {
            throw new IllegalArgumentException("Baidu apiUrl must be not empty.");
        }

        if (StringUtils.isBlank(imageUrl)) {
            throw new IllegalArgumentException("face imageUrl must be not empty.");
        }

        String requestUrl = String.format("%s?access_token=%s", apiUrl, accessToken);

        List<Map<String, String>> params = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        // imageData.put("image_type", "BASE64");
        imageData.put("image_type", "URL");
        imageData.put("image", imageUrl);
        params.add(imageData);
        String jsonParams = new JsonHelper().toJson(params);

        final String contentType = "application/json; charset=utf-8";
        RequestBody formBody = FormBody.create(MediaType.parse(contentType), jsonParams);
        Request.Builder request = new Request.Builder().url(requestUrl).post(formBody);

        return callApi(apiUrl, request);
    }

    public static BaiduResponse faceCompare(String accessToken, String apiUrl, String imageUrl, String imageUrl2) {
        if (StringUtils.isBlank(apiUrl)) {
            throw new IllegalArgumentException("Baidu apiUrl must be not empty.");
        }

        if (StringUtils.isBlank(imageUrl)) {
            throw new IllegalArgumentException("face imageUrl must be not empty.");
        }

        if (StringUtils.isBlank(imageUrl2)) {
            throw new IllegalArgumentException("face imageUrl2 must be not empty.");
        }

        String requestUrl = String.format("%s?access_token=%s", apiUrl, accessToken);

        List<Map<String, String>> params = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "URL");
        imageData.put("image", imageUrl);
        params.add(imageData);
        Map<String, String> imageData2 = new TreeMap<>();
        imageData2.put("image_type", "URL");
        imageData2.put("image", imageUrl2);
        params.add(imageData2);
        String jsonParams = new JsonHelper().toJson(params);

        final String contentType = "application/json; charset=utf-8";
        RequestBody formBody = FormBody.create(MediaType.parse(contentType), jsonParams);
        Request.Builder request = new Request.Builder().url(requestUrl).post(formBody);

        return callApi(apiUrl, request);
    }

    private static BaiduResponse callApi(String apiUrl, Request.Builder request) {
        BaiduResponse result = null;
        try (Response response = HttpClientBuilder.getClient().newCall(request.build()).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();

            System.out.println(bodyStr);

            if (response.isSuccessful()) {
                result = new JsonHelper().parseJson(bodyStr, BaiduResponse.class);
            } else {
                log.warn("[baidu] url:{} response:{} body:{}", apiUrl, response.code(), bodyStr);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        return result;
    }

}
